@@ -347,12 +347,21 @@ func (c *connection) failLeftRequestsWhenClose() {
 	// and closing the channel
 	c.incomingRequestsWG.Wait()
 
-	reqLen := len(c.incomingRequestsCh)
-	for i := 0; i < reqLen; i++ {
-		c.internalSendRequest(<-c.incomingRequestsCh)
+	ch := c.incomingRequestsCh
+	go func() {
+		// send a nil message to drain instead of
+		// closing the channel and causing a potential panic
+		//
+		// if other requests come in after the nil message
+		// then the RPC client will time out
+		ch <- nil
+	}()
+	for req := range ch {
+		if nil == req {
+			break // we have drained the requests
+		}
+		c.internalSendRequest(req)
 	}
-
-	close(c.incomingRequestsCh)
 }
 
 func (c *connection) run() {
@@ -367,12 +376,7 @@ func (c *connection) run() {
 		// all the accesses to the pendingReqs should be happened in this run loop thread,
 		// including the final cleanup, to avoid the issue
 		// https://github.com/apache/pulsar-client-go/issues/239
-		c.pendingLock.Lock()
-		for id, req := range c.pendingReqs {
-			req.callback(nil, errConnectionClosed)
-			delete(c.pendingReqs, id)
-		}
-		c.pendingLock.Unlock()
+		c.failPendingRequests(errConnectionClosed)
 		c.Close()
 	}()
 
@@ -536,7 +540,7 @@ func (c *connection) internalReceivedCommand(cmd *pb.BaseCommand, headersAndPayl
 		c.handleResponseError(cmd.GetError())
 
 	case pb.BaseCommand_SEND_ERROR:
-		c.handleSendError(cmd.GetError())
+		c.handleSendError(cmd.GetSendError())
 
 	case pb.BaseCommand_CLOSE_PRODUCER:
 		c.handleCloseProducer(cmd.GetCloseProducer())
@@ -697,6 +701,16 @@ func (c *connection) deletePendingRequest(requestID uint64) (*request, bool) {
 	return request, ok
 }
 
+func (c *connection) failPendingRequests(err error) bool {
+	c.pendingLock.Lock()
+	defer c.pendingLock.Unlock()
+	for id, req := range c.pendingReqs {
+		req.callback(nil, err)
+		delete(c.pendingReqs, id)
+	}
+	return true
+}
+
 func (c *connection) lastDataReceived() time.Time {
 	c.lastDataReceivedLock.Lock()
 	defer c.lastDataReceivedLock.Unlock()
@@ -747,24 +761,29 @@ func (c *connection) handleAuthChallenge(authChallenge *pb.CommandAuthChallenge)
 	c.writeCommand(baseCommand(pb.BaseCommand_AUTH_RESPONSE, cmdAuthResponse))
 }
 
-func (c *connection) handleSendError(cmdError *pb.CommandError) {
-	c.log.Warnf("Received send error from server: [%v] : [%s]", cmdError.GetError(), cmdError.GetMessage())
+func (c *connection) handleSendError(sendError *pb.CommandSendError) {
+	c.log.Warnf("Received send error from server: [%v] : [%s]", sendError.GetError(), sendError.GetMessage())
 
-	requestID := cmdError.GetRequestId()
+	producerID := sendError.GetProducerId()
 
-	switch *cmdError.Error {
+	switch sendError.GetError() {
 	case pb.ServerError_NotAllowedError:
-		request, ok := c.deletePendingRequest(requestID)
+		_, ok := c.deletePendingProducers(producerID)
 		if !ok {
 			c.log.Warnf("Received unexpected error response for request %d of type %s",
-				requestID, cmdError.GetError())
+				producerID, sendError.GetError())
 			return
 		}
 
-		errMsg := fmt.Sprintf("server error: %s: %s", cmdError.GetError(), cmdError.GetMessage())
-		request.callback(nil, errors.New(errMsg))
+		c.log.Warnf("server error: %s: %s", sendError.GetError(), sendError.GetMessage())
 	case pb.ServerError_TopicTerminatedError:
-		// TODO: no-op
+		_, ok := c.deletePendingProducers(producerID)
+		if !ok {
+			c.log.Warnf("Received unexpected error response for producer %d of type %s",
+				producerID, sendError.GetError())
+			return
+		}
+		c.log.Warnf("server error: %s: %s", sendError.GetError(), sendError.GetMessage())
 	default:
 		// By default, for transient error, let the reconnection logic
 		// to take place and re-establish the produce again
@@ -772,6 +791,17 @@ func (c *connection) handleSendError(cmdError *pb.CommandError) {
 	}
 }
 
+func (c *connection) deletePendingProducers(producerID uint64) (ConnectionListener, bool) {
+	c.listenersLock.Lock()
+	producer, ok := c.listeners[producerID]
+	if ok {
+		delete(c.listeners, producerID)
+	}
+	c.listenersLock.Unlock()
+
+	return producer, ok
+}
+
 func (c *connection) handleCloseConsumer(closeConsumer *pb.CommandCloseConsumer) {
 	consumerID := closeConsumer.GetConsumerId()
 	c.log.Infof("Broker notification of Closed consumer: %d", consumerID)
@@ -788,13 +818,7 @@ func (c *connection) handleCloseProducer(closeProducer *pb.CommandCloseProducer)
 	c.log.Infof("Broker notification of Closed producer: %d", closeProducer.GetProducerId())
 	producerID := closeProducer.GetProducerId()
 
-	c.listenersLock.Lock()
-	producer, ok := c.listeners[producerID]
-	if ok {
-		delete(c.listeners, producerID)
-	}
-	c.listenersLock.Unlock()
-
+	producer, ok := c.deletePendingProducers(producerID)
 	// did we find a producer?
 	if ok {
 		producer.ConnectionClosed()
@@ -830,10 +854,8 @@ func (c *connection) Close() {
 	c.closeOnce.Do(func() {
 		c.Lock()
 		cnx := c.cnx
-		// do not use changeState() since they share the same lock
-		c.setState(connectionClosed)
-		c.cond.Broadcast()
 		c.Unlock()
+		c.changeState(connectionClosed)
 
 		if cnx != nil {
 			_ = cnx.Close()
@@ -872,10 +894,8 @@ func (c *connection) Close() {
 }
 
 func (c *connection) changeState(state connectionState) {
-	c.Lock()
 	c.setState(state)
 	c.cond.Broadcast()
-	c.Unlock()
 }
 
 func (c *connection) getState() connectionState {
